@@ -45,6 +45,13 @@
 
 namespace batchedGemm {
 
+namespace trtllm {
+namespace gen {
+class CudaRunner;
+class GenCfg;
+}  // namespace gen
+}  // namespace trtllm
+
 namespace gemmGatedAct {
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -130,10 +137,6 @@ inline bool checkAndUpdateGemmGatedActOptions(gemmGatedAct::GemmGatedActOptions&
         "Unsupported output hidden tile size");
   }
 
-  if (options.mUseDeepSeekFp8) {
-    TLLM_CHECK_ERROR(hiddenSize % 256 == 0, "Output hidden size must be a multiple of 256");
-  }
-
   if (options.mDtypeC == tg::Dtype::E2m1 || options.mDtypeC == tg::Dtype::MxE4m3) {
     int const outHiddenSize = (options.mTransposeMmaOutput ? options.mM : options.mN) / 2;
     int const hiddenGranularity = 4 * tg::dtypeNumEltsPerSf(options.mDtypeC);
@@ -148,6 +151,21 @@ inline bool checkAndUpdateGemmGatedActOptions(gemmGatedAct::GemmGatedActOptions&
     return false;
   }
 
+  auto const validHiddenSize = options.mTransposeMmaOutput ? options.mValidM : options.mValidN;
+  if (options.mUseDeepSeekFp8) {
+    TLLM_CHECK_ERROR(hiddenSize % 256 == 0 && validHiddenSize % 256 == 0, "Hidden size (",
+                     hiddenSize, ") and valid hidden size (", validHiddenSize,
+                     ") must be a multiple of 256");
+  }
+
+  //
+  if (options.mUseShuffledMatrixA) {
+    auto const shuffleBlockSize = gemm::getShuffleBlockSize(options.mEpilogueTileM);
+    TLLM_CHECK_ERROR(
+        hiddenSize % (2 * shuffleBlockSize) == 0 && validHiddenSize % (2 * shuffleBlockSize) == 0,
+        "M/validM must be a multiple of 2 * shuffle block size (", 2 * shuffleBlockSize,
+        ") when useShuffledMatrixA");
+  }
   if (options.mNumSlicesForSplitK > 1) {
     TLLM_CHECK_ERROR(doesSplitKUseDsmem(options.mSplitK),
                      "Split-k GMEM and GemmGatedAct are not supported yet.");
@@ -163,11 +181,11 @@ inline bool checkAndUpdateGemmGatedActOptions(gemmGatedAct::GemmGatedActOptions&
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 
-inline std::string dumpOptions(GemmGatedActOptions const& options) {
+inline std::string dumpOptions(GemmGatedActOptions const& options, bool dumpRuntimeParams = true) {
   std::stringstream ss;
-  ss << gemm::dumpOptions(options) << ", ";
-  ss << "mActType=" << "gemmGatedAct::ActType(" << static_cast<int32_t>(options.mActType) << "),"
-     << std::endl;
+  ss << gemm::dumpOptions(options, dumpRuntimeParams) << ", ";
+  ss << "mActType="
+     << "gemmGatedAct::ActType(" << static_cast<int32_t>(options.mActType) << ")," << std::endl;
   ss << "mClampBeforeAct=" << options.mClampBeforeAct << "" << std::endl;
   return ss.str();
 }
@@ -179,19 +197,18 @@ inline std::string dumpOptions(GemmGatedActOptions const& options) {
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 
 struct GemmGatedActConfig {
-  // When TRT-LLM Gen is exported to the other frameworks, the TLLM_GEN_EXPORT_INTERFACE must be
-  // defined. In this case, the cubins will be loaded from the provided data and function name.
-  // Otherwise, the kernel will be loaded from the CudaRunner.
-#ifdef TLLM_GEN_EXPORT_INTERFACE
   uint8_t const* mData{nullptr};
-  uint32_t const mSize{0};
-  uint32_t const mSharedMemSize{0};
+  uint32_t mSize{0};
+  uint32_t mSharedMemSize{0};
   char const* mFunctionName{nullptr};
-  uint32_t const mNumThreadsPerCTA{0};
+  uint32_t mNumThreadsPerCTA{0};
   char const* mHash{nullptr};
-#else
+
+  std::string mGenCfgJsonStr{""};
+  char const* mExecPath{nullptr};
   trtllm::gen::CudaRunner* mCudaRunner{nullptr};
-#endif
+  trtllm::gen::GenCfg* mGenCfg{nullptr};
+  int32_t mInstanceIdx{0};
 
   GemmGatedActOptions mOptions{};
   gemm::SmVersion mSm{gemm::SmVersion::Sm100a};
