@@ -26,33 +26,10 @@ struct KernelParams {
   //
   //////////////////////////////////////////////////////////////////////////////////////////////////
 
-  // Maximum number of CTAs
+  // Maximum number of CTAs in the batch-token dimension.
   static constexpr int MaxNumCtas = 2048;
 
-  // NOTE: TMA out-of-bounds optimization for MoE padded tokens:
   //
-  // 1. 2D tensor [hidden, numPaddedTokens] with stride [1, hidden] at pointer p
-  // 2. Reshape to 3D: [hidden, tileN, numPaddedTokens] with stride [1, hidden, hidden]
-  // 3. Transform coordinates
-  // Originally:
-  //
-  //   Ptr (p               ) <-> Coord [:, 0               , ctaIdxY * tileN           ]
-  //
-  // Equals
-  //
-  //   Ptr (p - tileN*hidden) <-> Coord [:, tileN           , ctaIdxY * tileN           ]
-  //
-  // Equals
-  //
-  //   Ptr (p - tileN*hidden) <-> Coord [:, tileN - batchEnd, ctaIdxY * tileN + batchEnd]
-  //
-  // In effect, we load exactly `batchEnd` tokens from same global address, reducing wasted traffic.
-  //
-  // Due to the restrictions of TMA unit, the above operations requires the TMA descriptor and the
-  // underlying buffer be constructed differently:
-  // - Requires valid buffer at (p - tileN * hidden) - needs prepending `tileN` tokens.
-  // - TMA outermost dimension must be extended by `tileN` or loads will OOB in the rightmost side.
-
   // TMA descriptor for A.
   // Must be setup using gemm::buildNdTmaDescriptor with shapes and strides from
   // makeTmaShapeStrideAbc.
@@ -232,15 +209,15 @@ struct KernelParams {
   //   x_linear = x_linear.clamp(min=-limit, max=limit)
   float const* ptrClampLimit{nullptr};
 
-  // The alpha and beta for SwiGlu.
+  // The alpha and beta for SwiGlu or GeGlu.
   // Shape is [B]. One alpha and one beta per tensor in batch.
   // Alpha is 1.f if nullptr.
   // Beta is 0.f if nullptr.
-  // The formula:
+  // The formula for SwiGlu (for GeGlu, replace sigmoid with phi):
   //
   //   out_glu  = x_glu * torch.sigmoid(alpha * x_glu) * (x_linear + beta)
-  float const* ptrSwiGluAlpha{nullptr};
-  float const* ptrSwiGluBeta{nullptr};
+  float const* ptrGatedActAlpha{nullptr};
+  float const* ptrGatedActBeta{nullptr};
 
   // The K dimension. It is the hidden dimension of the input matrices.
   int32_t k;
